use std::{collections::HashMap, ops::DerefMut};

use dioxus::prelude::*;

#[component]
pub fn NewsCreate() -> Element {
    let groups = use_context::<crate::Groups>();
    let hours = use_context::<crate::Hours>();

    rsx! {
        div {
            class: "w-full max-w-2xl space-y-3",
            h1 { class: "text-xl font-bold text-primary", "Nieuw bericht" }
            form {
                class: "flex flex-col gap-y-5",
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Titel" }
                    },
                    input {
                        r#type: "text",
                        class: "input input-bordered w-full",
                    }
                }
                label {
                    class: "form-control w-full",
                    div {
                        class: "label",
                        span { class: "label-text", "Bericht" }
                    },
                    textarea {
                        class: "textarea textarea-bordered w-full",
                    }
                },
                TargetSelect { }
            }
        }
    }
}

#[derive(Eq, PartialEq, Clone, Copy, Debug)]
enum TargetKind {
    None,
    All,
    Group,
    Hourgroup,
    Hour,
    Member,
    Account,
    Day,
}

impl std::fmt::Display for TargetKind {
    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
        match self {
            TargetKind::All => write!(f, "all"),
            TargetKind::Group => write!(f, "group"),
            TargetKind::Hourgroup => write!(f, "hourgroup"),
            TargetKind::Hour => write!(f, "hour"),
            TargetKind::Member => write!(f, "member"),
            TargetKind::Account => write!(f, "account"),
            TargetKind::Day => write!(f, "day"),
            TargetKind::None => write!(f, "none"),
        }
    }
}

struct Target {
    kind: TargetKind,
    value: String,
}

impl TargetKind {
    fn from_string(input: &str) -> Self {
        match input {
            "all" => Self::All,
            "group" => Self::Group,
            "hourgroup" => Self::Hourgroup,
            "hour" => Self::Hour,
            "member" => Self::Member,
            "account" => Self::Account,
            "day" => Self::Day,
            _ => Self::None,
        }
    }
}

#[component]
fn TargetSelect() -> Element {
    let mut targets: Signal<HashMap<u32, Target>> = use_signal(|| {
        HashMap::from([(
            0,
            Target {
                kind: TargetKind::None,
                value: String::new(),
            },
        )])
    });

    let mut target_id = use_signal(|| 1);

    let target_ids = use_memo(move || {
        let mut filtered_targets = targets.read().iter().map(|x| *x.0).collect::<Vec<_>>();

        filtered_targets.sort_unstable();

        filtered_targets
    });

    rsx! {
        label {
            class: "form-control w-full",
            div {
                class: "label",
                span { class: "label-text", "Naar" }
            },
            for id in target_ids() {
                TargetEntry { key: "{id}", id, targets }
            }
            button {
                class: "btn btn-primary btn-outline btn-sm mt-3 mr-auto",
                r#type: "button",
                onclick: move |_| {
                    let id = target_id();
                    let target = Target {
                        kind: TargetKind::None,
                        value: String::new(),
                    };
                    targets.write().insert(id, target);
                    target_id += 1;

                },
                "Conditie toevoegen",
            }
        }
    }
}

#[component]
fn TargetEntry(mut targets: Signal<HashMap<u32, Target>>, id: u32) -> Element {
    let kind = use_memo(move || targets.read().get(&id).unwrap().kind);

    rsx! {
        div {
            class: "join w-full mt-3",
            select {
                class: "select select-bordered join-item w-full",
                oninput: move |event| {
                    let target_kind = TargetKind::from_string(&event.value());
                    targets.write().get_mut(&id).unwrap().kind = target_kind;
                },
                option { disabled: true, selected: true, value: "none",  "Selecteer een type" },
                option { value: "all", selected: kind() == TargetKind::All, "Iedereen" }
                option { value: "group", selected: kind() == TargetKind::Group, "Groep" }
                option { value: "day", selected: kind() == TargetKind::Day, "Dag" }
                option { value: "hour", selected: kind() == TargetKind::Hour, "Uur" }
                option { value: "hourgroup", selected: kind() == TargetKind::Hourgroup, "Lesgroep" }
                option { value: "member", selected: kind() == TargetKind::Member, "Lid" }
                option { value: "account", selected: kind() == TargetKind::Account, "Account" }
            }
            match kind() {
                _ => rsx! {
                    div { class: "input input-bordered w-full join-item" }
                }
            }
        }
    }
}
